const std = @import("std");

///
/// Non-cryptographic, hashing function suitable for scenarios where a quick and lightweight hash is needed.
///
/// Unlike Jenkins hashing, djb2 is not optimized for a specific sequence size.
///
pub fn djb2(comptime Int: type, bytes: []const u8) Int {
    var hash = @as(Int, @intCast(5381));

    for (bytes) |byte| {
        hash = ((hash << 5) +% hash) +% byte;
    }

    return hash;
}

///
/// Non-cryptographic hashing function that is designed to be fast over secure.
///
/// It is particularly effective for small to medium-sized keys, providing low collision rates and provides good
/// distribution properties.
///
pub fn jenkins(comptime Int: type, bytes: []const u8) Int {
    var hash = @as(Int, 0);

    for (bytes) |byte| {
        hash +%= byte;
        hash +%= (hash << 10);
        hash ^= (hash >> 6);
    }

    hash +%= (hash << 3);
    hash ^= (hash >> 11);
    hash +%= (hash << 15);

    return hash;
}
